!MNH_LIC Copyright 1995-2022 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
!#################
MODULE MODI_EMOIST
!#################
!
INTERFACE
!
#ifndef MNH_OPENACC
FUNCTION EMOIST(KRR,KRRI,PTHLM,PRM,PLOCPEXNM,PAMOIST,PSRCM) RESULT(PEMOIST)
#else
SUBROUTINE EMOIST(KRR,KRRI,PTHLM,PRM,PLOCPEXNM,PAMOIST,PSRCM,PEMOIST)
#endif
!
INTEGER                              :: KRR        ! number of moist var.
INTEGER                              :: KRRI       ! number of ice var.
!
REAL, DIMENSION(:,:,:),  INTENT(IN)  ::   PTHLM    ! Conservative pot. temperature
REAL, DIMENSION(:,:,:,:), INTENT(IN) ::   PRM      ! Mixing ratios, where
!                                    PRM(:,:,:,1) = conservative mixing ratio
REAL, DIMENSION(:,:,:),  INTENT(IN)  ::   PLOCPEXNM ! Lv(T)/Cp/Exner at time t-1
REAL, DIMENSION(:,:,:),  INTENT(IN)  ::   PAMOIST   ! Amoist
REAL, DIMENSION(:,:,:),  INTENT(IN)  ::   PSRCM     ! Normalized 2dn_order
                                                    ! moment s'r'c/2Sigma_s2
!
#ifndef MNH_OPENACC
REAL,DIMENSION(SIZE(PTHLM,1),SIZE(PTHLM,2),SIZE(PTHLM,3)):: PEMOIST ! result
#else
REAL,DIMENSION(:,:,:),  INTENT(OUT):: PEMOIST ! result
#endif
!
#ifndef MNH_OPENACC
END FUNCTION EMOIST
#else
END SUBROUTINE EMOIST
#endif
!
END INTERFACE
!
END MODULE MODI_EMOIST
!   ############################################################################
#ifndef MNH_OPENACC
FUNCTION EMOIST(KRR,KRRI,PTHLM,PRM,PLOCPEXNM,PAMOIST,PSRCM) RESULT(PEMOIST)
#else
SUBROUTINE EMOIST(KRR,KRRI,PTHLM,PRM,PLOCPEXNM,PAMOIST,PSRCM,PEMOIST)
#endif
!   ############################################################################
!
!      PURPOSE
!!     -------
!      EMOIST computes the coefficient Emoist in the flottability turbulent
!      flux. This coefficient relates the vertical flux of the virtual potential
!      temperature ( <Thv' W'> ) to the vertical flux of the conservative mixing
!      ratio ( <Rnp' W'> ). 
!
!!**   METHOD
!!     ------
!!     The virtual potential temperature perturbation is linearized in function
!!     of Thl' and Rnp'. The result is
!!        Thv'= ( ZA + ZC * Atheta * 2 * SRC ) Thl' 
!!             +( ZB + ZC * Amoist * 2 * SRC ) Rnp'
!!     From this relation, we can compute the verical turbulent fluxes.
!! 
!!     EXTERNAL
!!     --------
!!       NONE
!!
!!     IMPLICIT ARGUMENTS
!!     ------------------
!!       Module MODD_CST : contains physical constants.
!!         XRV, XRD  : R for water vapor and dry air
!!   
!!     REFERENCE
!!     ---------
!! 
!!       NONE
!!
!!
!!     AUTHOR
!!     ------
!!       Jean-Marie Carriere      * Meteo-France *
!!
!!     MODIFICATIONS
!!     -------------
!!       Original       20/03/95
!!
!!       J. Stein       Feb  28, 1996   optimization + Doctorization
!!       J. Stein       Spet 15, 1996   Amoist previously computed
!!       J.-P. Pinty    May  20, 2003   Improve EMOIST expression
!!       M.Moge         April, 2016     Use openACC directives to port the TURB part of Meso-NH on GPU
!!                  03/2021 (JL Redelsperger) Ocean model case 
!! 
!! ----------------------------------------------------------------------
!
!*       0. DECLARATIONS
!           ------------
USE MODD_CST
USE MODD_DYN_n, ONLY : LOCEAN

#ifdef MNH_OPENACC
USE MODE_MNH_ZWORK, ONLY: MNH_MEM_GET, MNH_MEM_POSITION_PIN, MNH_MEM_RELEASE
#endif
use mode_mppdb

IMPLICIT NONE
!
!*       0.1 declarations of arguments and result
!
!
INTEGER                              :: KRR        ! number of moist var.
INTEGER                              :: KRRI       ! number of ice var.
!
REAL, DIMENSION(:,:,:),  INTENT(IN)  ::   PTHLM    ! Conservative pot. temperature
REAL, DIMENSION(:,:,:,:), INTENT(IN) ::   PRM      ! Mixing ratios, where
!                                    PRM(:,:,:,1) = conservative mixing ratio
REAL, DIMENSION(:,:,:),  INTENT(IN)  ::   PLOCPEXNM ! Lv(T)/Cp/Exner at time t-1
REAL, DIMENSION(:,:,:),  INTENT(IN)  ::   PAMOIST   ! Amoist
REAL, DIMENSION(:,:,:),  INTENT(IN)  ::   PSRCM     ! Normalized 2dn_order
                                                    ! moment s'r'c/2Sigma_s2
!
#ifndef MNH_OPENACC
REAL,DIMENSION(SIZE(PTHLM,1),SIZE(PTHLM,2),SIZE(PTHLM,3)):: PEMOIST ! result
#else
REAL,DIMENSION(:,:,:),  INTENT(OUT):: PEMOIST ! result
#endif
!
!*       0.2 declarations of local variables
!
REAL,DIMENSION(:,:,:), pointer,contiguous :: ZA, ZRW
!                ZA = coeft A, ZRW = total mixing ratio rw 
REAL                                  :: ZDELTA  ! = Rv/Rd - 1
INTEGER                               :: JRR     ! moist loop counter
INTEGER                               :: JIU,JJU,JKU
INTEGER                               :: JI,JJ,JK
!
!---------------------------------------------------------------------------

!$acc data present( PTHLM, PRM, PLOCPEXNM, PAMOIST, PSRCM, PEMOIST )

if ( mppdb_initialized ) then
  !Check all in arrays
  call Mppdb_check( pthlm,     "Emoist beg:pthlm"     )
  call Mppdb_check( prm,       "Emoist beg:prm"       )
  call Mppdb_check( plocpexnm, "Emoist beg:plocpexnm" )
  call Mppdb_check( pamoist,   "Emoist beg:pamoist"   )
  call Mppdb_check( psrcm,     "Emoist beg:psrcm"     )
end if

JIU =  size( pthlm, 1 )
JJU =  size( pthlm, 2 )
JKU =  size( pthlm, 3 )

#ifndef MNH_OPENACC
allocate( za  (JIU,JJU,JKU) )
allocate( zrw (JIU,JJU,JKU ) )
#else
!Pin positions in the pools of MNH memory
CALL MNH_MEM_POSITION_PIN()

CALL MNH_MEM_GET( za,  JIU, JJU, JKU )
CALL MNH_MEM_GET( zrw, JIU, JJU, JKU )
#endif

!$acc data present( za, zrw )

!
!*       1. COMPUTE EMOIST
!           --------------
IF (LOCEAN) THEN
 IF ( KRR == 0 ) THEN                                ! Unsalted
!$acc kernels
   PEMOIST(:,:,:) = 0.
!$acc end kernels
ELSE
!$acc kernels
   PEMOIST(:,:,:) = 1.                              ! Salted case
!$acc end kernels
 END IF
!
ELSE
!
 IF ( KRR == 0 ) THEN                                ! dry case
!$acc kernels
   PEMOIST(1:JIU,1:JJU,1:JKU) = 0.
!$acc end kernels
 ELSE IF ( KRR == 1 ) THEN                           ! only vapor
!$acc kernels
  ZDELTA = (XRV/XRD) - 1.
  PEMOIST(1:JIU,1:JJU,1:JKU) = ZDELTA*PTHLM(1:JIU,1:JJU,1:JKU)
!$acc end kernels
 ELSE                                                ! liquid water & ice present
!$acc kernels
  ZDELTA = (XRV/XRD) - 1.
  ZRW(1:JIU,1:JJU,1:JKU) = PRM(1:JIU,1:JJU,1:JKU,1)
!$acc end kernels
!
  IF ( KRRI>0) THEN  ! rc and ri case
!$acc kernels      
   ZRW(1:JIU,1:JJU,1:JKU) = ZRW(1:JIU,1:JJU,1:JKU) + PRM(1:JIU,1:JJU,1:JKU,3)
   !$acc loop seq  
   DO JRR=5,KRR
      ZRW(1:JIU,1:JJU,1:JKU) = ZRW(1:JIU,1:JJU,1:JKU) + PRM(1:JIU,1:JJU,1:JKU,JRR)
   ENDDO
#ifdef MNH_COMPILER_NVHPC
   !$acc loop independent collapse(3)
#endif
   DO CONCURRENT ( JI=1:JIU,JJ=1:JJU,JK=1:JKU)
      ZA(JI,JJ,JK) = 1. + (                                    &  ! Compute A
           (1.+ZDELTA) * (PRM(JI,JJ,JK,1) - PRM(JI,JJ,JK,2) - PRM(JI,JJ,JK,4)) &
           -ZRW(JI,JJ,JK)                                                &
           )  /  (1. + ZRW(JI,JJ,JK))
   END DO !CONCURRENT  
  !
  !   Emoist = ZB + ZC * Amoist
  !   ZB is computed from line 1 to line 2
  !   ZC is computed from line 3 to line 5
  !   Amoist* 2 * SRC is computed at line 6
  !
    PEMOIST(1:JIU,1:JJU,1:JKU) = ZDELTA * (PTHLM(1:JIU,1:JJU,1:JKU) + PLOCPEXNM(1:JIU,1:JJU,1:JKU)*(               &
                                                    PRM(1:JIU,1:JJU,1:JKU,2)+PRM(1:JIU,1:JJU,1:JKU,4)))&
                            / (1. + ZRW(1:JIU,1:JJU,1:JKU))                                &
        +( PLOCPEXNM(1:JIU,1:JJU,1:JKU) * ZA(1:JIU,1:JJU,1:JKU)                                        &
               -(1.+ZDELTA) * (PTHLM(1:JIU,1:JJU,1:JKU) + PLOCPEXNM(1:JIU,1:JJU,1:JKU)*(               &
                                                    PRM(1:JIU,1:JJU,1:JKU,2)+PRM(1:JIU,1:JJU,1:JKU,4)))&
                            / (1. + ZRW(1:JIU,1:JJU,1:JKU))                                &
                            ) * PAMOIST(1:JIU,1:JJU,1:JKU) * 2. * PSRCM(1:JIU,1:JJU,1:JKU)
!$acc end kernels    
 ELSE
!$acc kernels
   !$acc loop seq
   DO JRR=3,KRR
      ZRW(1:JIU,1:JJU,1:JKU) = ZRW(1:JIU,1:JJU,1:JKU) + PRM(1:JIU,1:JJU,1:JKU,JRR)
   ENDDO
#ifdef MNH_COMPILER_NVHPC
   !$acc loop independent collapse(3)
#endif
   DO CONCURRENT ( JI=1:JIU,JJ=1:JJU,JK=1:JKU)
      ZA(JI,JJ,JK) = 1. + (                                    &  ! Compute ZA
           (1.+ZDELTA) * (PRM(JI,JJ,JK,1) - PRM(JI,JJ,JK,2)) &
           -ZRW(JI,JJ,JK)                                 &
           )  /  (1. + ZRW(JI,JJ,JK))
   END DO !CONCURRENT 
  !
  !   Emoist = ZB + ZC * Amoist
  !   ZB is computed from line 1 to line 2
  !   ZC is computed from line 3 to line 5
  !   Amoist* 2 * SRC is computed at line 6
  !
    PEMOIST(1:JIU,1:JJU,1:JKU) = ZDELTA * (PTHLM(1:JIU,1:JJU,1:JKU) + PLOCPEXNM(1:JIU,1:JJU,1:JKU)*PRM(1:JIU,1:JJU,1:JKU,2))   &
                            / (1. + ZRW(1:JIU,1:JJU,1:JKU))                                &
        +( PLOCPEXNM(1:JIU,1:JJU,1:JKU) * ZA(1:JIU,1:JJU,1:JKU)                                        &
               -(1.+ZDELTA) * (PTHLM(1:JIU,1:JJU,1:JKU) + PLOCPEXNM(1:JIU,1:JJU,1:JKU)*PRM(1:JIU,1:JJU,1:JKU,2))   &
                            / (1. + ZRW(1:JIU,1:JJU,1:JKU))                                &
                            ) * PAMOIST(1:JIU,1:JJU,1:JKU) * 2. * PSRCM(1:JIU,1:JJU,1:JKU)
!$acc end kernels
  END IF
 END IF
!
END IF
! acc end kernels

if ( mppdb_initialized ) then
  !Check all out arrays
  call Mppdb_check( pemoist, "Emoist end:pemoist" )
end if

!$acc end data

#ifndef MNH_OPENACC
deallocate( za, zrw )
#else
!Release all memory allocated with MNH_MEM_GET calls since last call to MNH_MEM_POSITION_PIN
CALL MNH_MEM_RELEASE()
#endif

!$acc end data

!---------------------------------------------------------------------------
!
#ifndef MNH_OPENACC
END FUNCTION EMOIST
#else
END SUBROUTINE EMOIST
#endif
