!MNH_LIC Copyright 1994-2021 CNRS, Meteo-France and Universite Paul Sabatier
!MNH_LIC This is part of the Meso-NH software governed by the CeCILL-C licence
!MNH_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!MNH_LIC for details. version 1.
!-----------------------------------------------------------------
MODULE MODI_ICE4_RIMLTC
INTERFACE
SUBROUTINE ICE4_RIMLTC(LDSOFT, PCOMPUTE, &
                       &PEXN, PLVFACT, PLSFACT, &
                       &PT, &
                       &PTHT, PRIT, &
                       &PRIMLTC_MR, PB_TH, PB_RC, PB_RI)
IMPLICIT NONE
LOGICAL,                  INTENT(IN)    :: LDSOFT
REAL, DIMENSION(:),       INTENT(IN)    :: PCOMPUTE
REAL, DIMENSION(:),       INTENT(IN)    :: PEXN     ! Exner function
REAL, DIMENSION(:),       INTENT(IN)    :: PLVFACT  ! L_v/(Pi_ref*C_ph)
REAL, DIMENSION(:),       INTENT(IN)    :: PLSFACT  ! L_s/(Pi_ref*C_ph)
REAL, DIMENSION(:),       INTENT(IN)    :: PT       ! Temperature
REAL, DIMENSION(:),       INTENT(IN)    :: PTHT     ! Theta at t
REAL, DIMENSION(:),       INTENT(IN)    :: PRIT     ! Cloud ice at t
REAL, DIMENSION(:),       INTENT(OUT)   :: PRIMLTC_MR ! Mixing ratio change due to cloud ice melting
REAL, DIMENSION(:),       INTENT(INOUT) :: PB_TH
REAL, DIMENSION(:),       INTENT(INOUT) :: PB_RC
REAL, DIMENSION(:),       INTENT(INOUT) :: PB_RI
END SUBROUTINE ICE4_RIMLTC
END INTERFACE
END MODULE MODI_ICE4_RIMLTC
SUBROUTINE ICE4_RIMLTC(LDSOFT, PCOMPUTE, &
                       &PEXN, PLVFACT, PLSFACT, &
                       &PT, &
                       &PTHT, PRIT, &
                       &PRIMLTC_MR, PB_TH, PB_RC, PB_RI)
!!
!!**  PURPOSE
!!    -------
!!      Computes the RIMLTC process
!!
!!    AUTHOR
!!    ------
!!      S. Riette from the splitting of rain_ice source code (nov. 2014)
!!
!!    MODIFICATIONS
!!    -------------
!!
!
!
!*      0. DECLARATIONS
!          ------------
!
USE MODD_CST,       ONLY: XTT
USE MODD_PARAM_ICE, ONLY: LFEEDBACKT
!
#ifdef MNH_OPENACC
USE MODE_MNH_ZWORK, ONLY: MNH_MEM_GET, MNH_MEM_POSITION_PIN, MNH_MEM_RELEASE
#endif
USE MODE_MPPDB
!
IMPLICIT NONE
!
!*       0.1   Declarations of dummy arguments :
!
LOGICAL,                  INTENT(IN)    :: LDSOFT
REAL, DIMENSION(:),       INTENT(IN)    :: PCOMPUTE
REAL, DIMENSION(:),       INTENT(IN)    :: PEXN     ! Exner function
REAL, DIMENSION(:),       INTENT(IN)    :: PLVFACT  ! L_v/(Pi_ref*C_ph)
REAL, DIMENSION(:),       INTENT(IN)    :: PLSFACT  ! L_s/(Pi_ref*C_ph)
REAL, DIMENSION(:),       INTENT(IN)    :: PT       ! Temperature
REAL, DIMENSION(:),       INTENT(IN)    :: PTHT     ! Theta at t
REAL, DIMENSION(:),       INTENT(IN)    :: PRIT     ! Cloud ice at t
REAL, DIMENSION(:),       INTENT(OUT)   :: PRIMLTC_MR ! Mixing ratio change due to cloud ice melting
REAL, DIMENSION(:),       INTENT(INOUT) :: PB_TH
REAL, DIMENSION(:),       INTENT(INOUT) :: PB_RC
REAL, DIMENSION(:),       INTENT(INOUT) :: PB_RI
!
!*       0.2  declaration of local variables
!
INTEGER                         :: ISIZE
INTEGER                         :: JL
#ifndef MNH_OPENACC
REAL, DIMENSION(:), allocatable :: ZMASK
#else
REAL, DIMENSION(:), POINTER, CONTIGUOUS :: ZMASK
#endif

!$acc data present(PCOMPUTE,PEXN,PLVFACT,PLSFACT,PT, &
!$acc&                PTHT,PRIT,PRIMLTC_MR,PB_TH,PB_RC,PB_RI)
!
!-------------------------------------------------------------------------------
IF (MPPDB_INITIALIZED) THEN
  !Check all IN arrays
  CALL MPPDB_CHECK(PCOMPUTE,"ICE4_RIMLTC beg:PCOMPUTE")
  CALL MPPDB_CHECK(PEXN,"ICE4_RIMLTC beg:PEXN")
  CALL MPPDB_CHECK(PLVFACT,"ICE4_RIMLTC beg:PLVFACT")
  CALL MPPDB_CHECK(PLSFACT,"ICE4_RIMLTC beg:PLSFACT")
  CALL MPPDB_CHECK(PT,"ICE4_RIMLTC beg:PT")
  CALL MPPDB_CHECK(PTHT,"ICE4_RIMLTC beg:PTHT")
  CALL MPPDB_CHECK(PRIT,"ICE4_RIMLTC beg:PRIT")
  !Check all INOUT arrays
  CALL MPPDB_CHECK(PB_TH,"ICE4_RIMLTC beg:PB_TH")
  CALL MPPDB_CHECK(PB_RC,"ICE4_RIMLTC beg:PB_RC")
  CALL MPPDB_CHECK(PB_RI,"ICE4_RIMLTC beg:PB_RI")
END IF

isize = Size( pcompute )

#ifndef MNH_OPENACC
allocate( zmask(isize) )
#else
!Pin positions in the pools of MNH memory
CALL MNH_MEM_POSITION_PIN()

CALL MNH_MEM_GET( zmask, isize )

!$acc data present(ZMASK)
#endif
!
!*       7.1    cloud ice melting
!
!$acc kernels
PRIMLTC_MR(:)=0.
IF(.NOT. LDSOFT) THEN
  DO JL=1, ISIZE
    ZMASK(JL)=MAX(0., -SIGN(1., -PRIT(JL))) * & ! PRIT(:)>0.
             &MAX(0., -SIGN(1., XTT-PT(JL))) * & ! PT(:)>XTT
             &PCOMPUTE(JL)
    PRIMLTC_MR(JL)=PRIT(JL) * ZMASK(JL)
  ENDDO

  IF(LFEEDBACKT) THEN
    !Limitation due to 0 crossing of temperature
    DO JL=1, ISIZE
      PRIMLTC_MR(JL)=MIN(PRIMLTC_MR(JL), MAX(0., (PTHT(JL)-XTT/PEXN(JL)) / (PLSFACT(JL)-PLVFACT(JL))))
    ENDDO
  ENDIF
ENDIF
DO JL=1, ISIZE
  PB_RC(JL) = PB_RC(JL) + PRIMLTC_MR(JL)
  PB_RI(JL) = PB_RI(JL) - PRIMLTC_MR(JL)
  PB_TH(JL) = PB_TH(JL) - PRIMLTC_MR(JL)*(PLSFACT(JL)-PLVFACT(JL))
ENDDO
!$acc end kernels

!$acc end data

#ifdef MNH_OPENACC
  !Release all memory allocated with MNH_MEM_GET calls since last call to MNH_MEM_POSITION_PIN
  CALL MNH_MEM_RELEASE()
#endif

IF (MPPDB_INITIALIZED) THEN
  !Check all INOUT arrays
  CALL MPPDB_CHECK(PB_TH,"ICE4_RIMLTC end:PB_TH")
  CALL MPPDB_CHECK(PB_RC,"ICE4_RIMLTC end:PB_RC")
  CALL MPPDB_CHECK(PB_RI,"ICE4_RIMLTC end:PB_RI")
  !Check all OUT arrays
  CALL MPPDB_CHECK(PRIMLTC_MR,"ICE4_RIMLTC end:PRIMLTC_MR")
END IF

!$acc end data

END SUBROUTINE ICE4_RIMLTC
